@@ -19,7 +19,7 @@ class RegisterContextLinux_mips64
     : public lldb_private::RegisterInfoInterface
 {
 public:
-    RegisterContextLinux_mips64(const lldb_private::ArchSpec &target_arch);
+    RegisterContextLinux_mips64(const lldb_private::ArchSpec &target_arch, bool msa_present = true);
 
     size_t
     GetGPRSize() const override;
@@ -30,9 +30,13 @@ public:
     uint32_t
     GetRegisterCount () const override;
 
+    uint32_t
+    GetUserRegisterCount () const override;
+
 private:
     const lldb_private::RegisterInfo *m_register_info_p;
     uint32_t m_register_info_count;
+    uint32_t m_user_register_count;
 };
 
 #endif
